@@ -2,7 +2,7 @@
  * Copyright 2011      INRIA Saclay
  * Copyright 2012      Ecole Normale Superieure
  *
- * Use of this software is governed by the GNU LGPLv2.1 license
+ * Use of this software is governed by the MIT license
  *
  * Written by Sven Verdoolaege, INRIA Saclay - Ile-de-France,
  * Parc Club Orsay Universite, ZAC des vignes, 4 rue Jacques Monod,
@@ -192,6 +192,18 @@ const char *isl_local_space_get_dim_name(__isl_keep isl_local_space *ls,
 	return ls ? isl_space_get_dim_name(ls->dim, type, pos) : NULL;
 }
 
+int isl_local_space_has_dim_id(__isl_keep isl_local_space *ls,
+	enum isl_dim_type type, unsigned pos)
+{
+	return ls ? isl_space_has_dim_id(ls->dim, type, pos) : -1;
+}
+
+__isl_give isl_id *isl_local_space_get_dim_id(__isl_keep isl_local_space *ls,
+	enum isl_dim_type type, unsigned pos)
+{
+	return ls ? isl_space_get_dim_id(ls->dim, type, pos) : NULL;
+}
+
 __isl_give isl_aff *isl_local_space_get_div(__isl_keep isl_local_space *ls,
 	int pos)
 {
@@ -207,6 +219,9 @@ __isl_give isl_aff *isl_local_space_get_div(__isl_keep isl_local_space *ls,
 	if (isl_int_is_zero(ls->div->row[pos][0]))
 		isl_die(isl_local_space_get_ctx(ls), isl_error_invalid,
 			"expression of div unknown", return NULL);
+	if (!isl_local_space_is_set(ls))
+		isl_die(isl_local_space_get_ctx(ls), isl_error_invalid,
+			"cannot represent divs of map spaces", return NULL);
 
 	aff = isl_aff_alloc(isl_local_space_copy(ls));
 	if (!aff)
@@ -424,7 +439,7 @@ static int cmp_row(isl_int *row_i, isl_int *row_j, int i, int j,
 
 /* Call cmp_row for divs in a matrix.
  */
-static int mat_cmp_row(__isl_keep isl_mat *div, int i, int j)
+int isl_mat_cmp_div(__isl_keep isl_mat *div, int i, int j)
 {
 	return cmp_row(div->row[i], div->row[j], i, j, div->n_row, div->n_col);
 }
@@ -504,7 +519,7 @@ __isl_give isl_mat *isl_merge_divs(__isl_keep isl_mat *div1,
 		expand_row(div, k, div1, i, exp1);
 		expand_row(div, k + 1, div2, j, exp2);
 
-		cmp = mat_cmp_row(div, k, k + 1);
+		cmp = isl_mat_cmp_div(div, k, k + 1);
 		if (cmp == 0) {
 			exp1[i++] = k;
 			exp2[j++] = k;
@@ -530,6 +545,27 @@ __isl_give isl_mat *isl_merge_divs(__isl_keep isl_mat *div1,
 	return div;
 }
 
+/* Swap divs "a" and "b" in "ls".
+ */
+__isl_give isl_local_space *isl_local_space_swap_div(
+	__isl_take isl_local_space *ls, int a, int b)
+{
+	int offset;
+
+	ls = isl_local_space_cow(ls);
+	if (!ls)
+		return NULL;
+	if (a < 0 || a >= ls->div->n_row || b < 0 || b >= ls->div->n_row)
+		isl_die(isl_local_space_get_ctx(ls), isl_error_invalid,
+			"index out of bounds", return isl_local_space_free(ls));
+	offset = ls->div->n_col - ls->div->n_row;
+	ls->div = isl_mat_swap_cols(ls->div, offset + a, offset + b);
+	ls->div = isl_mat_swap_rows(ls->div, a, b);
+	if (!ls->div)
+		return isl_local_space_free(ls);
+	return ls;
+}
+
 /* Construct a local space that contains all the divs in either
  * "ls1" or "ls2".
  */
@@ -685,14 +721,13 @@ __isl_give isl_local_space *isl_local_space_substitute_equalities(
 	unsigned total;
 	unsigned n_div;
 
-	ls = isl_local_space_cow(ls);
 	if (!ls || !eq)
 		goto error;
 
 	total = isl_space_dim(eq->dim, isl_dim_all);
 	if (isl_local_space_dim(ls, isl_dim_all) != total)
 		isl_die(isl_local_space_get_ctx(ls), isl_error_invalid,
-			"dimensions don't match", goto error);
+			"spaces don't match", goto error);
 	total++;
 	n_div = eq->n_div;
 	for (i = 0; i < eq->n_eq; ++i) {
@@ -703,6 +738,12 @@ __isl_give isl_local_space *isl_local_space_substitute_equalities(
 		for (k = 0; k < ls->div->n_row; ++k) {
 			if (isl_int_is_zero(ls->div->row[k][1 + j]))
 				continue;
+			ls = isl_local_space_cow(ls);
+			if (!ls)
+				goto error;
+			ls->div = isl_mat_cow(ls->div);
+			if (!ls->div)
+				goto error;
 			isl_seq_elim(ls->div->row[k] + 1, eq->eq[i], j, total,
 					&ls->div->row[k][0]);
 			normalize_div(ls, k);
@@ -717,6 +758,58 @@ error:
 	return NULL;
 }
 
+/* Plug in the affine expressions "subs" of length "subs_len" (including
+ * the denominator and the constant term) into the variable at position "pos"
+ * of the "n" div expressions starting at "first".
+ *
+ * Let i be the dimension to replace and let "subs" be of the form
+ *
+ *	f/d
+ *
+ * Any integer division starting at "first" with a non-zero coefficient for i,
+ *
+ *	floor((a i + g)/m)
+ *
+ * is replaced by
+ *
+ *	floor((a f + d g)/(m d))
+ */
+__isl_give isl_local_space *isl_local_space_substitute_seq(
+	__isl_take isl_local_space *ls,
+	enum isl_dim_type type, unsigned pos, isl_int *subs, int subs_len,
+	int first, int n)
+{
+	int i;
+	isl_int v;
+
+	if (n == 0)
+		return ls;
+	ls = isl_local_space_cow(ls);
+	if (!ls)
+		return NULL;
+	ls->div = isl_mat_cow(ls->div);
+	if (!ls->div)
+		return isl_local_space_free(ls);
+
+	if (first + n > ls->div->n_row)
+		isl_die(isl_local_space_get_ctx(ls), isl_error_invalid,
+			"index out of bounds", return isl_local_space_free(ls));
+
+	pos += isl_local_space_offset(ls, type);
+
+	isl_int_init(v);
+	for (i = first; i < ls->div->n_row; ++i) {
+		if (isl_int_is_zero(ls->div->row[i][1 + pos]))
+			continue;
+		isl_seq_substitute(ls->div->row[i], pos, subs,
+			ls->div->n_col, subs_len, v);
+		normalize_div(ls, i);
+	}
+	isl_int_clear(v);
+
+	return ls;
+}
+
 /* Plug in "subs" for dimension "type", "pos" in the integer divisions
  * of "ls".
  *
@@ -736,9 +829,6 @@ __isl_give isl_local_space *isl_local_space_substitute(
 	__isl_take isl_local_space *ls,
 	enum isl_dim_type type, unsigned pos, __isl_keep isl_aff *subs)
 {
-	int i;
-	isl_int v;
-
 	ls = isl_local_space_cow(ls);
 	if (!ls || !subs)
 		return isl_local_space_free(ls);
@@ -751,24 +841,8 @@ __isl_give isl_local_space *isl_local_space_substitute(
 			"cannot handle divs yet",
 			return isl_local_space_free(ls));
 
-	pos += isl_local_space_offset(ls, type);
-
-	isl_int_init(v);
-	for (i = 0; i < ls->div->n_row; ++i) {
-		if (isl_int_is_zero(ls->div->row[i][1 + pos]))
-			continue;
-		isl_int_set(v, ls->div->row[i][1 + pos]);
-		isl_int_set_si(ls->div->row[i][1 + pos], 0);
-		isl_seq_combine(ls->div->row[i] + 1,
-				subs->v->el[0], ls->div->row[i] + 1,
-				v, subs->v->el + 1, subs->v->size - 1);
-		isl_int_mul(ls->div->row[i][0],
-			    ls->div->row[i][0], subs->v->el[0]);
-		normalize_div(ls, i);
-	}
-	isl_int_clear(v);
-
-	return ls;
+	return isl_local_space_substitute_seq(ls, type, pos, subs->v->el,
+					    subs->v->size, 0, ls->div->n_row);
 }
 
 int isl_local_space_is_named_or_nested(__isl_keep isl_local_space *ls,
@@ -914,7 +988,7 @@ int *isl_local_space_get_active(__isl_keep isl_local_space *ls, isl_int *l)
 	ctx = isl_local_space_get_ctx(ls);
 	total = isl_local_space_dim(ls, isl_dim_all);
 	active = isl_calloc_array(ctx, int, total);
-	if (!active)
+	if (total && !active)
 		return NULL;
 
 	for (i = 0; i < total; ++i)
@@ -974,3 +1048,85 @@ __isl_give isl_basic_map *isl_local_space_lifting(
 
 	return lifting;
 }
+
+/* Compute the preimage of "ls" under the function represented by "ma".
+ * In other words, plug in "ma" in "ls".  The result is a local space
+ * that is part of the domain space of "ma".
+ *
+ * If the divs in "ls" are represented as
+ *
+ *	floor((a_i(p) + b_i x + c_i(divs))/n_i)
+ *
+ * and ma is represented by
+ *
+ *	x = D(p) + F(y) + G(divs')
+ *
+ * then the resulting divs are
+ *
+ *	floor((a_i(p) + b_i D(p) + b_i F(y) + B_i G(divs') + c_i(divs))/n_i)
+ *
+ * We first copy over the divs from "ma" and then
+ * we add the modified divs from "ls".
+ */
+__isl_give isl_local_space *isl_local_space_preimage_multi_aff(
+	__isl_take isl_local_space *ls, __isl_take isl_multi_aff *ma)
+{
+	int i;
+	isl_space *space;
+	isl_local_space *res = NULL;
+	int n_div_ls, n_div_ma;
+	isl_int f, c1, c2, g;
+
+	ma = isl_multi_aff_align_divs(ma);
+	if (!ls || !ma)
+		goto error;
+	if (!isl_space_is_range_internal(ls->dim, ma->space))
+		isl_die(isl_local_space_get_ctx(ls), isl_error_invalid,
+			"spaces don't match", goto error);
+
+	n_div_ls = isl_local_space_dim(ls, isl_dim_div);
+	n_div_ma = ma->n ? isl_aff_dim(ma->p[0], isl_dim_div) : 0;
+
+	space = isl_space_domain(isl_multi_aff_get_space(ma));
+	res = isl_local_space_alloc(space, n_div_ma + n_div_ls);
+	if (!res)
+		goto error;
+
+	if (n_div_ma) {
+		isl_mat_free(res->div);
+		res->div = isl_mat_copy(ma->p[0]->ls->div);
+		res->div = isl_mat_add_zero_cols(res->div, n_div_ls);
+		res->div = isl_mat_add_rows(res->div, n_div_ls);
+		if (!res->div)
+			goto error;
+	}
+
+	isl_int_init(f);
+	isl_int_init(c1);
+	isl_int_init(c2);
+	isl_int_init(g);
+
+	for (i = 0; i < ls->div->n_row; ++i) {
+		if (isl_int_is_zero(ls->div->row[i][0])) {
+			isl_int_set_si(res->div->row[n_div_ma + i][0], 0);
+			continue;
+		}
+		isl_seq_preimage(res->div->row[n_div_ma + i], ls->div->row[i],
+				ma, 0, 0, n_div_ma, n_div_ls, f, c1, c2, g, 1);
+		normalize_div(res, n_div_ma + i);
+	}
+
+	isl_int_clear(f);
+	isl_int_clear(c1);
+	isl_int_clear(c2);
+	isl_int_clear(g);
+
+	isl_local_space_free(ls);
+	isl_multi_aff_free(ma);
+	return res;
+error:
+	isl_local_space_free(ls);
+	isl_multi_aff_free(ma);
+	isl_local_space_free(res);
+	return NULL;
+}
