@@ -1207,6 +1207,12 @@ static struct isl_basic_map *remove_duplicate_constraints(
  *	-floor(e/m) = ceil(-e/m) = floor((-e + m - 1)/m)
  *
  *
+ * Note that one of the div constraints may have been eliminated
+ * due to being redundant with respect to the constraint that is
+ * being modified by this function.  The modified constraint may
+ * no longer imply this div constraint, so we add it back to make
+ * sure we do not lose any information.
+ *
  * We skip integral divs, i.e., those with denominator 1, as we would
  * risk eliminating the div from the div constraints.  We do not need
  * to handle those divs here anyway since the div constraints will turn
@@ -1258,6 +1264,10 @@ static __isl_give isl_basic_map *eliminate_unit_divs(
 				isl_int_sub_ui(bmap->ineq[j][0],
 					bmap->ineq[j][0], 1);
 			}
+
+			bmap = isl_basic_map_extend_constraints(bmap, 0, 1);
+			if (isl_basic_map_add_div_constraint(bmap, i, s) < 0)
+				return isl_basic_map_free(bmap);
 		}
 	}
 
@@ -2435,11 +2445,14 @@ int isl_basic_set_plain_is_disjoint(__isl_keep isl_basic_set *bset1,
 }
 
 /* Are "map1" and "map2" obviously disjoint?
- *
- * If one of them is empty or if they live in different spaces (ignoring
- * parameters), then they are clearly disjoint.
  *
  * If they have different parameters, then we skip any further tests.
+ * In particular, the outcome of the subsequent calls to
+ * isl_space_tuple_match may be affected by the different parameters
+ * in nested spaces.
+ *
+ * If one of them is empty or if they live in different spaces (assuming
+ * they have the same parameters), then they are clearly disjoint.
  *
  * If they are obviously equal, but not obviously empty, then we will
  * not be able to detect if they are disjoint.
@@ -2466,6 +2479,11 @@ int isl_map_plain_is_disjoint(__isl_keep isl_map *map1,
 	if (disjoint < 0 || disjoint)
 		return disjoint;
 
+	match = isl_space_match(map1->dim, isl_dim_param,
+				map2->dim, isl_dim_param);
+	if (match < 0 || !match)
+		return match < 0 ? -1 : 0;
+
 	match = isl_space_tuple_match(map1->dim, isl_dim_in,
 				map2->dim, isl_dim_in);
 	if (match < 0 || !match)
@@ -2476,11 +2494,6 @@ int isl_map_plain_is_disjoint(__isl_keep isl_map *map1,
 	if (match < 0 || !match)
 		return match < 0 ? -1 : 1;
 
-	match = isl_space_match(map1->dim, isl_dim_param,
-				map2->dim, isl_dim_param);
-	if (match < 0 || !match)
-		return match < 0 ? -1 : 0;
-
 	intersect = isl_map_plain_is_equal(map1, map2);
 	if (intersect < 0 || intersect)
 		return intersect < 0 ? -1 : 0;
