@@ -11,7 +11,7 @@
 #define ISL_PIPLIB_H
 
 #include <isl/ctx.h>
-#include <isl/int.h>
+#include <isl_int.h>
 #include <isl/map.h>
 #ifndef ISL_PIPLIB
 #error "no piplib"
