@@ -12,11 +12,15 @@
 #include <isl/aff.h>
 #include <isl/ast.h>
 #include <isl/printer.h>
+#include <isl/val.h>
 
 #undef BASE
 #define BASE id
 #include <print_templ.c>
 #undef BASE
+#define BASE val
+#include <print_templ.c>
+#undef BASE
 #define BASE space
 #include <print_templ.c>
 #undef BASE
