@@ -1,5 +1,5 @@
 for (int c0 = 0; c0 < Ncl; c0 += 1)
-  if (c0 >= 1 && Ncl >= c0 + 2) {
+  if (Ncl >= c0 + 2 && c0 >= 1) {
     S(c0, 28);
   } else if (c0 == 0) {
     S(0, 26);
